package eu.ztsh.lfr.core.impl.files;

import eu.ztsh.lfr.config.DataProperties;
import eu.ztsh.lfr.model.DataRow;
import eu.ztsh.lfr.model.FileProcessingException;
import eu.ztsh.lfr.model.Temperatures;
import eu.ztsh.lfr.model.events.FileModifiedEvent;
import eu.ztsh.lfr.model.events.FileProcessedEvent;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.context.event.EventListener;
import org.springframework.scheduling.annotation.Async;
import org.springframework.stereotype.Service;

import java.io.IOException;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.util.concurrent.TimeUnit;
import java.util.function.Predicate;
import java.util.stream.Stream;

@Service
public class FileLoadingService {

    private static final Logger log = LoggerFactory.getLogger(FileLoadingService.class);
    private final DataProperties dataProperties;
    private final ApplicationEventPublisher eventPublisher;

    @Autowired
    public FileLoadingService(DataProperties dataProperties, ApplicationEventPublisher eventPublisher) {
        this.dataProperties = dataProperties;
        this.eventPublisher = eventPublisher;
    }

    @Async
    @EventListener(FileModifiedEvent.class)
    public void onFileModified() {
        var start = System.currentTimeMillis();
        log.debug("Processing file");
        final Temperatures model = new Temperatures();
        try (final Stream<String> lines = Files.lines(Paths.get(dataProperties.fileUrl()))) {
            lines.parallel()
                .filter(Predicate.not(String::isBlank))
                .map(line -> {
                    try {
                        var parts = line.split(";");
                        return new DataRow(parts[0], Integer.parseInt(parts[1].split("-")[0]), Double.parseDouble(parts[2]));
                    } catch (Exception e) {
                        throw new FileProcessingException("Error in line %s".formatted(line), e);
                    }
                })
                .forEach(model::addData);
        } catch (IOException e) {
            throw new FileProcessingException("File error", e);
        }
        var millis = System.currentTimeMillis() - start;
        log.atInfo().setMessage("Elapsed time: {}:{}").addArgument(TimeUnit.MILLISECONDS.toMinutes(millis))
            .addArgument(() -> String.format("%02d", TimeUnit.MILLISECONDS.toSeconds(millis) % 60))
            .log();
        eventPublisher.publishEvent(new FileProcessedEvent(model.getAverages()));
    }

}
